==> l10n_ar_api-2.3.1.tar.gz: l10n_ar_api-2.3.1/l10n_ar_api/afip_webservices/wsaa/tokens.py <==
# -*- coding: utf-8 -*-
import pytz
from datetime import datetime
import time
from lxml import etree
from M2Crypto import BIO, SMIME
import sys


class AccessRequerimentToken(object):
    
    def __init__(self, service, tz=pytz.utc):
        self._service = service
        self._timezone = tz

    def _create_tra(self):
        """
        uniqueId: Entero de 32 bits sin signo que junto con generationTime identifica el
        requerimiento.
        
        generationTime: Momento en que fue generado el requerimiento. La tolerancia de aceptacion
        sera de hasta 24 horas previas al requerimiento de acceso
        
        expirationTime: Momento en el que expira la solicitud. La tolerancia de aceptacion sera de
        hasta 24 horas posteriores al requerimiento de acceso.  
        
        service: Identificacion del WSN para el cual se solicita el TA.
        
        :return: Ticket de Requerimiento de Acceso
        """
        
        ticket_request = etree.Element('loginTicketRequest')
        ticket_request.set('version', '1.0')
        header = etree.SubElement(ticket_request, 'header')

        # uniqueId
        uniqueid = etree.SubElement(header, 'uniqueId')
        # Traemos la fecha actual
        timestamp = int(time.mktime(datetime.now().timetuple()))
        uniqueid.text = str(timestamp)
        
        # generationTime
        # Generamos el token con media hora atrasada
        tsgen = datetime.fromtimestamp(timestamp-1800)
        tsgen = pytz.utc.localize(tsgen).astimezone(self.timezone)
        gentime = etree.SubElement(header, 'generationTime')
        gentime.text = tsgen.isoformat()    

        # expirationTime
        # El token vence 4.5 horas despues
        tsexp = datetime.fromtimestamp(timestamp+14400)
        tsexp = pytz.utc.localize(tsexp).astimezone(self.timezone)
        exptime = etree.SubElement(header, 'expirationTime')
        exptime.text = tsexp.isoformat()
        exptime.tail = None

        # service
        serv = etree.SubElement(ticket_request, 'service')
        serv.text = self.service

        return etree.tostring(ticket_request)

    def sign_tra(self, private_key, certificate):

        smime = SMIME.SMIME()
        ks = BIO.MemoryBuffer(private_key.encode('ascii'))
        cs = BIO.MemoryBuffer(certificate.encode('ascii'))
        bf = BIO.MemoryBuffer(self._create_tra().decode('ascii').encode('ascii'))
        out = BIO.MemoryBuffer()

        try:
            smime.load_key_bio(ks, cs)
        except Exception:
            raise Exception('Error en el formato del certificado o clave privada')

        sbf = smime.sign(bf)
        smime.write(out, sbf)

        head, body, end = out.read().split(b'\n\n')
        return body
    
    @property
    def service(self):
        return self._service

    @property
    def timezone(self):
        return self._timezone


class AccessToken(object):
    
    """ Ejemplo de token:
    
    <?xml version="1.0" encoding="UTF8"?>
    <loginTicketResponse version="1.0">
    <header>
        <source>cn=wsaa,o=afip,c=ar,serialNumber=CUIT 33693450239</source>
        <destination>cn=srv1,ou=facturacion,o=empresa s.a.,c=ar,serialNumber=CUIT 30123456789</destination>
        <uniqueId>383953094</uniqueId>
        <generationTime>20011231T12:00:0203:00</generationTime>
        <expirationTime>20020101T00:00:0203:00</expirationTime>
    </header>
    <credentials>
        <token>cES0SSuWIIPlfe5/dLtb0Qeg2jQuvYuuSEDOrz+w2EnAQiEeS86gzYf7ehiU3UaYit5FRb9z/3zq</token>
        <sign>a6QSSZBgLf0TTcktSNteeSg3qXsMVjo/F5py/Gtw7xucTrUWbsrVCdIoGE8Cm1bixpuVPlr58k6n</sign>
    </credentials>
    </loginTicketResponse>
    """
    
    def __init__(self):
        # Header
        self.source = None
        self.destination = None
        self.uniqueId = None
        self.generation_time = None
        self.expiration_time = None

        # Credentials
        self.token = None
        self.sign = None
        
    def create_token_from_login(self, login_fault):
        """
        Asigna los atributos al token desde el login
        :param login_fault: XML con el login autorizado 
        """

        if sys.version_info[0] >= 3:
            login_fault_tree = etree.fromstring(login_fault.encode('utf-8'))
        else:
            login_fault_tree = etree.fromstring(str(login_fault))

        try:
            
            self.source = login_fault_tree.find('header/source').text
            self.destination = login_fault_tree.find('header/destination').text
            self.generation_time = login_fault_tree.find('header/generationTime').text
            self.expiration_time = login_fault_tree.find('header/expirationTime').text
            
            self.token = login_fault_tree.find('credentials/token').text
            self.sign = login_fault_tree.find('credentials/sign').text
        
        except AttributeError:
            raise AttributeError("Error al generar el token de acceso")

==> l10n_ar_api-2.9.1.tar.gz: l10n_ar_api-2.9.1/l10n_ar_api/afip_webservices/wsaa/tokens.py <==
# -*- coding: utf-8 -*-

import base64
from datetime import datetime
from lxml import etree
import pytz
import sys
import time

if sys.version_info[:2] >= (3, 10):
    from cryptography.hazmat.primitives.serialization import load_pem_private_key, pkcs7, Encoding
    from cryptography.hazmat.primitives.asymmetric import padding
    from cryptography.hazmat.primitives import hashes
    from cryptography.x509 import load_pem_x509_certificate
else:
    from M2Crypto import BIO, SMIME


class AccessRequerimentToken(object):
    
    def __init__(self, service, tz=pytz.utc):
        self._service = service
        self._timezone = tz

    def _create_tra(self):
        """
        uniqueId: Entero de 32 bits sin signo que junto con generationTime identifica el
        requerimiento.
        
        generationTime: Momento en que fue generado el requerimiento. La tolerancia de aceptacion
        sera de hasta 24 horas previas al requerimiento de acceso
        
        expirationTime: Momento en el que expira la solicitud. La tolerancia de aceptacion sera de
        hasta 24 horas posteriores al requerimiento de acceso.  
        
        service: Identificacion del WSN para el cual se solicita el TA.
        
        :return: Ticket de Requerimiento de Acceso
        """
        
        ticket_request = etree.Element('loginTicketRequest')
        ticket_request.set('version', '1.0')
        header = etree.SubElement(ticket_request, 'header')

        # uniqueId
        uniqueid = etree.SubElement(header, 'uniqueId')
        # Traemos la fecha actual
        timestamp = int(time.mktime(datetime.now().timetuple()))
        uniqueid.text = str(timestamp)
        
        # generationTime
        # Generamos el token con media hora atrasada
        tsgen = datetime.fromtimestamp(timestamp-1800)
        tsgen = pytz.utc.localize(tsgen).astimezone(self.timezone)
        gentime = etree.SubElement(header, 'generationTime')
        gentime.text = tsgen.isoformat()    

        # expirationTime
        # El token vence 4.5 horas despues
        tsexp = datetime.fromtimestamp(timestamp+14400)
        tsexp = pytz.utc.localize(tsexp).astimezone(self.timezone)
        exptime = etree.SubElement(header, 'expirationTime')
        exptime.text = tsexp.isoformat()
        exptime.tail = None

        # service
        serv = etree.SubElement(ticket_request, 'service')
        serv.text = self.service

        return etree.tostring(ticket_request)
    
    def sign_tra(self, private_key, certificate):
        tra_data = self._create_tra()
        pkey = private_key.encode('ascii')
        crt = certificate.encode('ascii')

        if sys.version_info[:2] >= (3, 10):
            try:
                key = load_pem_private_key(pkey, password=None)
            except Exception as e:
                raise ValueError("Formato de clave privada inválido") from e

            try:
                cert = load_pem_x509_certificate(crt)
            except Exception as e:
                raise ValueError("Formato de certificado inválido") from e

            try:
                signed_tra_data = base64.b64encode(pkcs7.PKCS7SignatureBuilder().set_data(
                    tra_data
                ).add_signer(
                    cert, key, hashes.SHA256(), rsa_padding=padding.PKCS1v15()
                ).sign(
                    Encoding.DER, [pkcs7.PKCS7Options.Binary]
                )).decode()

                return signed_tra_data
            except Exception as e:
                raise ValueError("Error firmando el TRA") from e

        else:
            smime = SMIME.SMIME()
            ks = BIO.MemoryBuffer(pkey)
            cs = BIO.MemoryBuffer(crt)
            bf = BIO.MemoryBuffer(tra_data)
            out = BIO.MemoryBuffer()

            try:
                smime.load_key_bio(ks, cs)
            except Exception:
                raise Exception('Error en el formato del certificado o clave privada')

            sbf = smime.sign(bf)
            smime.write(out, sbf)

            head, body, end = out.read().split(b'\n\n')
            return body
    
    @property
    def service(self):
        return self._service

    @property
    def timezone(self):
        return self._timezone


class AccessToken(object):
    
    """ Ejemplo de token:
    
    <?xml version="1.0" encoding="UTF8"?>
    <loginTicketResponse version="1.0">
    <header>
        <source>cn=wsaa,o=afip,c=ar,serialNumber=CUIT 33693450239</source>
        <destination>cn=srv1,ou=facturacion,o=empresa s.a.,c=ar,serialNumber=CUIT 30123456789</destination>
        <uniqueId>383953094</uniqueId>
        <generationTime>20011231T12:00:0203:00</generationTime>
        <expirationTime>20020101T00:00:0203:00</expirationTime>
    </header>
    <credentials>
        <token>cES0SSuWIIPlfe5/dLtb0Qeg2jQuvYuuSEDOrz+w2EnAQiEeS86gzYf7ehiU3UaYit5FRb9z/3zq</token>
        <sign>a6QSSZBgLf0TTcktSNteeSg3qXsMVjo/F5py/Gtw7xucTrUWbsrVCdIoGE8Cm1bixpuVPlr58k6n</sign>
    </credentials>
    </loginTicketResponse>
    """
    
    def __init__(self):
        # Header
        self.source = None
        self.destination = None
        self.uniqueId = None
        self.generation_time = None
        self.expiration_time = None

        # Credentials
        self.token = None
        self.sign = None
        
    def create_token_from_login(self, login_fault):
        """
        Asigna los atributos al token desde el login
        :param login_fault: XML con el login autorizado 
        """

        if sys.version_info[0] >= 3:
            login_fault_tree = etree.fromstring(login_fault.encode('utf-8'))
        else:
            login_fault_tree = etree.fromstring(str(login_fault))

        try:
            
            self.source = login_fault_tree.find('header/source').text
            self.destination = login_fault_tree.find('header/destination').text
            self.generation_time = login_fault_tree.find('header/generationTime').text
            self.expiration_time = login_fault_tree.find('header/expirationTime').text
            
            self.token = login_fault_tree.find('credentials/token').text
            self.sign = login_fault_tree.find('credentials/sign').text
        
        except AttributeError:
            raise AttributeError("Error al generar el token de acceso")
